#define CHECK_DIST
#include <iomanip>
void PadRes(Int_t module = 3, Int_t row = 16)
{
//--Set Style
  // Axis
  gStyle->SetLabelFont(22,"X");
  gStyle->SetLabelFont(22,"Y");
  // Title offset
  gStyle->SetTitleOffset(1.0, "X");
  gStyle->SetTitleOffset(1.2, "Y");
  // Pad grid
  gStyle->SetPadGridX(kTRUE);
  gStyle->SetPadGridY(kTRUE);
  // Title
  gStyle->SetTitleFont(22, "X");
  gStyle->SetTitleFont(22, "Y");
  gStyle->SetTitleFont(22, "");
  // -----------
  // Input files
  // -----------
  TList* fileList = new TList();
  //                           " file name"      drift length [cm]
  fileList->Add(new dataFile( "../Data/r17319.root" ,           5      ));
  fileList->Add(new dataFile( "../Data/r17367.root" ,          10      ));
  fileList->Add(new dataFile( "../Data/r17324.root" ,          15      ));
  fileList->Add(new dataFile( "../Data/r17364.root" ,          20      ));
  fileList->Add(new dataFile( "../Data/r17327.root" ,          25      ));
  fileList->Add(new dataFile( "../Data/r17359.root" ,          30      ));
  fileList->Add(new dataFile( "../Data/r17331.root" ,          35      ));
  fileList->Add(new dataFile( "../Data/r17356.root" ,          40      ));
  fileList->Add(new dataFile( "../Data/r17340.root" ,          45      ));
  fileList->Add(new dataFile( "../Data/r17352.root" ,          50      ));

  Int_t kNfile = fileList->GetEntries();

  // ---------------
  // Loop over runs
  // ---------------
  const Int_t kNp = kNfile;
  Double_t xdata[kNp], ydata[kNp], dxdata[kNp], dydata[kNp];
  Double_t x, y, dx, dy;

  for (Int_t i=0; i<kNp; i++) {
    Char_t* fname = (dynamic_cast<dataFile*>(fileList->At(i)))->getFileName();
    Int_t   dlen  = (dynamic_cast<dataFile*>(fileList->At(i)))->getDriftLength();
    Process(fname, dlen, module, row, x, y, dx, dy);
    
    xdata[i] = x; ydata[i] = y*y; dxdata[i] = dx; dydata[i] = 2*y*dy; 
  }

  // ---------------
  // Plot PRF vs z
  // ---------------
  TCanvas *c1 = new TCanvas("c1","",800,500);
  c1->cd();

  TGraphErrors *grp = new TGraphErrors(kNp,xdata,ydata,dxdata,dydata);
  stringstream titlestr;
  titlestr << "Pad Response (Module" << module << " Row" << row << ")" << ends;
  grp->SetTitle(titlestr.str().data());
  grp->GetHistogram()->GetXaxis()->SetLimits(0.,550.);
  grp->GetHistogram()->GetXaxis()->SetTitle("Drift Length: z [mm]");
  grp->GetHistogram()->GetYaxis()->SetTitle("#sigma_{PR}(0)^{2} [mm^{2}]");
  grp->Draw("ap");
  grp->SetMarkerColor(1);
  grp->SetMarkerStyle(21);
  grp->SetMinimum(0.);
  Double_t ymax = 1.5;
  grp->SetMaximum(ymax);
  TF1 fun("fun","[0]**2+[1]**2*x");
  fun.SetLineStyle(2);
  fun.SetLineColor(2);

  fun.SetParameter(0,0.1);
  fun.SetParameter(1,0.01);
  grp->Fit(&fun);
  Double_t sigma0  = fun.GetParameter(0) * 1.e3;             // [#mum]
  Double_t dsigma0 = fun.GetParError (0) * 1.e3;             // [#mum]
  Double_t cd      = fun.GetParameter(1) * sqrt(10.) * 1e3;  // [#mum/#sqrt{cm}]
  Double_t dcd     = fun.GetParError (1) * sqrt(10.) * 1e3;  // [#mum/#sqrt{cm}]
  Double_t chi2    = fun.GetChisquare();
  Int_t    ndf     = fun.GetNDF();

  TPaveText *pt = new TPaveText(330,8*ymax/10, 530.,8*ymax/10+1.8*ymax/10);

  pt->SetTextFont(132);
  pt->SetTextAlign(12);
  stringstream stext;
  stringstream stext;
  stext << "#chi^{2}/ndf = " << setprecision(3) << chi2 << "/" << ndf << ends;
  pt->AddText(stext.str().data());
  stext.str("");
  stext.clear();
  stext << "#sigma_{PR}(0) = " << setw(6) << setprecision(4) << sigma0 
        << " #pm "             << setw(3) << setprecision(2) << dsigma0 
        << " [#mum]"           << ends;
  pt->AddText(stext.str().data());
  stext.str("");
  stext.clear();
  stext << "C_{D} = " << setw(6) << setprecision(4) << cd
        << " #pm "    << setw(3) << setprecision(2) << dcd
        << " [#mum/#sqrt{cm}]" << ends;
  pt->AddText(stext.str().data());
  pt->Draw();

  TLatex *fitfun = new TLatex;
  fitfun->SetTextFont(132);
  fitfun->SetTextAlign(12);
  fitfun->DrawLatex(50.,ymax/2, "#sigma_{PR}^{2} = #sigma_{PR}(0)^{2}+(C_{D}^{2}) z");

#if 1
  // save plot as pdf file
  stringstream ofile;
  ofile << "PadRes_Module" << module << "_Row" << row << ".pdf"<< ends; 
  c1->Print(ofile.str().data());
#endif
}

void Process(Char_t* fname, Int_t dlen, Int_t module, Int_t row, 
             Double_t &x, Double_t &y, Double_t &dx, Double_t &dy)
{

  TFile *hfp = new TFile(fname);
#ifdef CHECK_DIST
  TDirectory *last = gDirectory;
  stringstream padresstr;
  padresstr << "c_" << dlen << ends;
  TCanvas *cp = new TCanvas(padresstr.str().data(),"",400,400);
  cp->cd();
#endif
  TNtupleD* hPadRes = dynamic_cast<TNtupleD*>(hfp->Get("MyRootFileProcessor/clrs"));
  stringstream cut;
  cut << "module==" << module << "&&" 
      << "row=="    << row    << ends;
  hPadRes->Draw("normchg:drphi>>htmp(100,-4,4,100,0,1.1)",cut.str().data(),"goff");
  TH2D *hq = static_cast<TH2D *>(htmp);
  hq->FitSlicesY();
  stringstream hslicestr;
  hslicestr << "htmp_1" << ends;
  TH1D *hslice = static_cast<TH1D *>(gROOT->FindObject(hslicestr.str().data()));

  //TF1 func("user","[0]*exp(-pow((x-[1])/[2],2)/2.)/sqrt(2*3.141592*[2])+[3]",-8.,+8.);
  TF1 func("user","[0]*exp(-pow((x-[1])/[2],2)/2.)/sqrt(2*3.141592*[2])",-8.,+8.);
  func.SetParameter(0,1.);
  func.SetParameter(1,0.);
  func.SetParameter(2,1.);
  //func.SetParameter(3,0.);
  hslice->Rebin(1);
  hslice->Fit("user","","",-2.,+2.);
  y  = hslice->GetFunction("user")->GetParameter(2);
  hslice->Fit("user","","",-5*y,5*y);

  x  = dlen;
  y  = hslice->GetFunction("user")->GetParameter(2);
  dx = 0.1; //FIXME
  dy = hslice->GetFunction("user")->GetParError(2);
  const Int_t kcm2mm = 10;
  x  *= kcm2mm;
  dx *= kcm2mm;
#ifdef CHECK_DIST
  hslice->SetMinimum(-0.1);
  hslice->SetMaximum(1.2);
  hslice->SetMarkerStyle(4);
  hslice->SetMarkerSize(0.7);
  hslice->SetMarkerColor(2);
  hslice->SetLineColor(2);
  hq->Draw();
  hslice->Draw("pe same");
  func.Draw("same");
  last->cd();
#endif
}

class dataFile : public TObject
{
  public: 
	  dataFile (Char_t* name, Int_t len) {
	    fname = name;
            flen  = len;
	  }

	  Char_t* getFileName()    { return fname; }
	  Int_t   getDriftLength() { return flen;  }

  private:
	  Char_t* fname; // input file name
	  Int_t    flen; // drift length [cm]
};
