#ifndef Runinfo_H
#define Runinfo_H
#include <iostream>
#include <map>

using namespace std;

class Runinfo {
public:
  // ----------
  // Getters
  // ----------
  static Runinfo *GetInstancePtr()
  {
    if (!gInstancePtr) gInstancePtr = new Runinfo;
    return gInstancePtr;
  }

  inline double GetDlength  (int run)
  {
    if (fRunMap.find(run) == fRunMap.end()) return 1.e10;
    else                                    return fRunMap[run].getDlength();
  }

  inline double GetBfield   (int run)
  {
    if (fRunMap.find(run) == fRunMap.end()) return 1.e10;
    else                                    return fRunMap[run].getBfield();
  }

  inline double GetMomentum(int run)
  {
    if (fRunMap.find(run) == fRunMap.end()) return 1.e10;
    else                                    return fRunMap[run].getMomentum();
  }

  // ---------
  // Utility
  // ---------
  inline void   RegisterRun(int run, int b, double p, double d)
  {
    if (fRunMap.find(run) == fRunMap.end()) {
      fRunMap[run] = RunCond(b,p,d);
    } else {
      cerr << "run" << run << " already there!" << endl;;
    }
  }

private:
  Runinfo()
  {
    RegisterRun(6953, 0, 5, 10);
    RegisterRun(6957, 0, 5, 15);
    RegisterRun(6958, 0, 5, 20);
    RegisterRun(6972, 0, 5, 25);
    RegisterRun(6973, 0, 5, 30);
    RegisterRun(6974, 0, 5, 35);
    RegisterRun(6975, 0, 5, 40);
    RegisterRun(6976, 0, 5, 45);
    RegisterRun(6977, 0, 5, 50);

    RegisterRun(7049, 1, 5, 10);
    RegisterRun(7046, 1, 5, 15);
    RegisterRun(7051, 1, 5, 20);
    RegisterRun(7052, 1, 5, 25);
    RegisterRun(7053, 1, 5, 25);
    RegisterRun(7059, 1, 5, 30);
    RegisterRun(7061, 1, 5, 35);
    RegisterRun(7065, 1, 5, 40);
    RegisterRun(7020, 1, 5, 45);
    RegisterRun(7023, 1, 5, 50);

    RegisterRun(17319, 1, 5,  5);
    RegisterRun(17324, 1, 5, 15);
    RegisterRun(17327, 1, 5, 25);
    RegisterRun(17331, 1, 5, 35);
    RegisterRun(17340, 1, 5, 45);
    RegisterRun(17344, 1, 5, 55);
    RegisterRun(17352, 1, 5, 50);
    RegisterRun(17356, 1, 5, 40);
    RegisterRun(17359, 1, 5, 30);
    RegisterRun(17364, 1, 5, 20);
    RegisterRun(17367, 1, 5, 10);
  }

private:
  // -------------
  // private class
  // -------------
  class RunCond {
  private:
    int    _bfield;
    double _momentum;
    double _dlength;

  public:
    RunCond(int b = 1, double p = 5., double d = 0.)
           : _bfield(b), _momentum(p), _dlength(d) {}

    inline int    getBfield  () const { return _bfield;   } // [T]
    inline double getDlength () const { return _dlength;  } // [cm]
    inline double getMomentum() const { return _momentum; } // [GeV]
  };

private:
  // -------------
  // Data members
  // -------------
  map<int, RunCond>  fRunMap;      // run map
  static Runinfo    *gInstancePtr; // singleton pointer
};

Runinfo *Runinfo::gInstancePtr = 0;
#endif
